@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import functools
 from inspect import Parameter, signature
 
@@ -5,22 +7,22 @@ from pathlib import Path
 from typing import (
     Any,
     Callable,
-    Dict,
     Iterable,
-    List,
     Literal,
     Optional,
+    Protocol,
     Type,
     Union,
     get_args,
     get_origin,
     get_type_hints,
 )
+from typing_extensions import Self
 from loguru import logger
+from ream.fs import FS
 
-import serde
+import serde.prelude as serde
 
-from ream.actors.base import BaseActor
 from ream.helper import orjson_dumps
 from serde.helper import JsonSerde
 
@@ -30,7 +32,7 @@ NoneType = type(None)
 class JLSerdeCache:
     @staticmethod
     def file(
-        cache_args: Optional[List[str]] = None,
+        cache_args: Optional[list[str]] = None,
         cache_key: Optional[Callable[[dict], bytes]] = None,
         filename: Optional[Union[str, Callable[..., str]]] = None,
         compression: Optional[Literal["gz", "bz2", "lz4"]] = None,
@@ -47,13 +49,14 @@ class JLSerdeCache:
             compression=compression,
             mem_persist=mem_persist,
             cache_attr=cache_attr,
+            fileext="jl",
         )
 
 
 class PickleSerdeCache:
     @staticmethod
     def file(
-        cache_args: Optional[List[str]] = None,
+        cache_args: Optional[list[str]] = None,
         cache_key: Optional[Callable[[dict], bytes]] = None,
         filename: Optional[Union[str, Callable[..., str]]] = None,
         compression: Optional[Literal["gz", "bz2", "lz4"]] = None,
@@ -69,16 +72,47 @@ class PickleSerdeCache:
             compression=compression,
             mem_persist=mem_persist,
             cache_attr=cache_attr,
+            fileext="pkl",
+        )
+
+
+class ClsSerdeCache:
+    """A cache that uses the save/load methods of a class as deser/ser functions"""
+
+    def __init__(self, cls: type[SaveLoadProtocol], ext: str = ".dat"):
+        self.cls = cls
+        self.ext = ext
+
+    def file(
+        self,
+        cache_args: Optional[list[str]] = None,
+        cache_key: Optional[Callable[[dict], bytes]] = None,
+        filename: Optional[Union[str, Callable[..., str]]] = None,
+        compression: Optional[Literal["gz", "bz2", "lz4"]] = None,
+        mem_persist: bool = False,
+        cache_attr: str = "_cache",
+    ):
+        return Cache.file(
+            ser=self.cls.save,
+            deser=self.cls.load,
+            cache_args=cache_args,
+            cache_key=cache_key,
+            filename=filename,
+            compression=compression,
+            mem_persist=mem_persist,
+            cache_attr=cache_attr,
+            fileext=self.ext,
         )
 
 
 class Cache:
     jl = JLSerdeCache
     pickle = PickleSerdeCache
+    cls = ClsSerdeCache
 
     @staticmethod
     def mem(
-        cache_args: Optional[List[str]] = None,
+        cache_args: Optional[list[str]] = None,
         cache_key: Optional[Callable[[dict], bytes]] = None,
         cache_attr: str = "_cache",
     ):
@@ -87,7 +121,7 @@ class Cache:
         Note: It does not support function with variable number of arguments.
 
         Args:
-            cache_args: List of arguments to use for the cache key. If None, all arguments are used.
+            cache_args: list of arguments to use for the cache key. If None, all arguments are used.
             cache_key: Function to use to generate the cache key. If None, the default is used. The default function
                 only support arguments of types str, int, bool, and None.
             cache_attr: Name of the attribute to use to store the cache in the instance.
@@ -122,21 +156,24 @@ class Cache:
     def file(
         ser: Callable[[Any, Path], None],
         deser: Callable[[Path], Any],
-        cache_args: Optional[List[str]] = None,
+        cache_args: Optional[list[str]] = None,
         cache_key: Optional[Callable[[dict], bytes]] = None,
         filename: Optional[Union[str, Callable[..., str]]] = None,
         compression: Optional[Literal["gz", "bz2", "lz4"]] = None,
         mem_persist: bool = False,
         cache_attr: str = "_cache",
+        fileext: Optional[str] = None,
     ) -> Callable:
-        """Decorator to cache the result of a function to a file.
+        """Decorator to cache the result of a function to a file. The function must
+        be a method of a class that has trait `HasWorkingFsTrait` so that we can determine
+        the directory to store the cached file.
 
         Note: It does not support function with variable number of arguments.
 
         Args:
             ser: A function to serialize the output of the function to a file.
             deser: A function to deserialize the output of the function from a file.
-            cache_args: List of arguments to use for the cache key. If None, all arguments are used.
+            cache_args: list of arguments to use for the cache key. If None, all arguments are used.
             cache_key: Function to use to generate the cache key. If None, the default is used. The default function
                 only support arguments of types str, int, bool, and None.
             filename: Filename to use for the cache file. If None, the name of the function is used. If it is a function,
@@ -145,11 +182,14 @@ class Cache:
                 this option has no effect if the filename is a function.
             mem_persist: If True, the cache will also be stored in memory. This is a combination of mem and file cache.
             cache_attr: Name of the attribute to use to store the cache in the instance.
+            fileext: Extension of the file to use if the filename is None (thus is derived automatically from the function name).
         """
 
         def wrapper_fn(func):
             if filename is None:
                 filename2 = func.__name__
+                if fileext is not None:
+                    filename2 += f".{fileext}"
                 if compression is not None:
                     filename2 += f".{compression}"
             else:
@@ -157,6 +197,11 @@ class Cache:
                 if isinstance(filename2, str) and compression is not None:
                     assert filename2.endswith(compression)
 
+            if isinstance(filename2, str):
+                assert (
+                    filename2.rfind(".") != -1
+                ), f"Caching to a file without extension is not supported. Got: {filename2}"
+
             cache_args_helper = CacheArgsHelper(func)
             if cache_args is not None:
                 cache_args_helper.keep_args(cache_args)
@@ -169,17 +214,21 @@ class Cache:
                 )
 
             @functools.wraps(func)
-            def fn(self: BaseActor, *args, **kwargs):
+            def fn(self: HasWorkingFsTrait, *args, **kwargs):
                 fs = self.get_working_fs()
 
                 if isinstance(filename2, str):
                     cache_filename = filename2
                 else:
                     cache_filename = filename2(*args, **kwargs)
+                    assert (
+                        cache_filename.rfind(".") != -1
+                    ), f"Caching to a file without extension is not supported. Got: {cache_filename}"
 
                 cache_file = fs.get(
                     cache_filename, key=keyfn(*args, **kwargs), save_key=True
                 )
+
                 if not cache_file.exists():
                     with fs.acquire_write_lock(), cache_file.reserve_and_track() as fpath:
                         output = func(self, *args, **kwargs)
@@ -202,9 +251,9 @@ class CacheArgsHelper:
     """
 
     def __init__(self, func: Callable):
-        self.args: Dict[str, Parameter] = {}
+        self.args: dict[str, Parameter] = {}
         try:
-            self.argtypes: Dict[str, Optional[Type]] = get_type_hints(func)
+            self.argtypes: dict[str, Optional[Type]] = get_type_hints(func)
         except TypeError:
             logger.error(
                 "Cannot get type hints for function {}. "
@@ -224,13 +273,13 @@ class CacheArgsHelper:
             next(iter(self.args)) == "self"
         ), "The first argument of the method must be self, an instance of BaseActor"
         self.args.pop("self")
-        self.argnames: List[str] = list(self.args.keys())
+        self.argnames: list[str] = list(self.args.keys())
         self.cache_args = self.argnames
 
     def keep_args(self, names: Iterable[str]) -> None:
         self.cache_args = list(names)
 
-    def get_cache_argtypes(self) -> Dict[str, Optional[Type]]:
+    def get_cache_argtypes(self) -> dict[str, Optional[Type]]:
         return {name: self.argtypes[name] for name in self.cache_args}
 
     def ensure_auto_cache_key_friendly(self):
@@ -288,3 +337,38 @@ class CacheArgsHelper:
     def get_args_as_tuple(self, *args, **kwargs) -> tuple:
         # TODO: improve me!
         return tuple(self.get_args(*args, **kwargs).values())
+
+
+class HasWorkingFsTrait(Protocol):
+    def get_working_fs(self) -> FS:
+        ...
+
+
+class Cacheable:
+    """A class that implement HasWorkingFSTrait so it can be used with @Cache.file decorator."""
+
+    def __init__(self, workdir: Path):
+        self.workdir = FS(workdir)
+
+    def get_working_fs(self) -> FS:
+        return self.workdir
+
+
+class SaveLoadProtocol(Protocol):
+    def save(self, file: Path) -> None:
+        ...
+
+    @classmethod
+    def load(cls, file: Path) -> Self:
+        ...
+
+
+def unwrap_cache_decorators(cls: type, methods: list[str] | None = None):
+    """Decorator to disable caching decorator by unwrap all methods of a class."""
+    for method in methods or dir(cls):
+        fn = getattr(cls, method)
+        iswrapped = hasattr(fn, "__wrapped__")
+        while hasattr(fn, "__wrapped__"):
+            fn = getattr(fn, "__wrapped__")  # type: ignore
+        if iswrapped:
+            setattr(cls, method, fn)
